@@ -27,6 +27,7 @@ import copy
 
 from .. import build
 from .. import dependencies
+from .. import programs
 from .. import mesonlib
 from .. import mlog
 from ..compilers import LANGUAGES_USING_LDFLAGS
@@ -49,6 +50,12 @@ if T.TYPE_CHECKING:
 # Assembly files cannot be unitified and neither can LLVM IR files
 LANGS_CANT_UNITY = ('d', 'fortran', 'vala')
 
+class RegenInfo:
+    def __init__(self, source_dir, build_dir, depfiles):
+        self.source_dir = source_dir
+        self.build_dir = build_dir
+        self.depfiles = depfiles
+
 class TestProtocol(enum.Enum):
 
     EXITCODE = 0
@@ -66,14 +73,15 @@ class TestProtocol(enum.Enum):
             return cls.GTEST
         elif string == 'rust':
             return cls.RUST
-        raise MesonException('unknown test format {}'.format(string))
+        raise MesonException(f'unknown test format {string}')
 
     def __str__(self) -> str:
-        if self is self.EXITCODE:
+        cls = type(self)
+        if self is cls.EXITCODE:
             return 'exitcode'
-        elif self is self.GTEST:
+        elif self is cls.GTEST:
             return 'gtest'
-        elif self is self.RUST:
+        elif self is cls.RUST:
             return 'rust'
         return 'tap'
 
@@ -99,20 +107,18 @@ class InstallData:
         self.strip_bin = strip_bin
         self.install_umask = install_umask
         self.targets: T.List[TargetInstallData] = []
-        self.headers: 'InstallType' = []
-        self.man: 'InstallType' = []
-        self.data: 'InstallType' = []
-        self.po_package_name: str = ''
-        self.po = []
+        self.headers: T.List[InstallDataBase] = []
+        self.man: T.List[InstallDataBase] = []
+        self.data: T.List[InstallDataBase] = []
         self.install_scripts: T.List[ExecutableSerialisation] = []
-        self.install_subdirs: 'InstallSubdirsType' = []
+        self.install_subdirs: T.List[SubdirInstallData] = []
         self.mesonintrospect = mesonintrospect
         self.version = version
 
 class TargetInstallData:
     def __init__(self, fname: str, outdir: str, aliases: T.Dict[str, str], strip: bool,
                  install_name_mappings: T.Dict, rpath_dirs_to_remove: T.Set[bytes],
-                 install_rpath: str, install_mode: 'FileMode', optional: bool = False):
+                 install_rpath: str, install_mode: 'FileMode', subproject: str, optional: bool = False):
         self.fname = fname
         self.outdir = outdir
         self.aliases = aliases
@@ -121,15 +127,28 @@ class TargetInstallData:
         self.rpath_dirs_to_remove = rpath_dirs_to_remove
         self.install_rpath = install_rpath
         self.install_mode = install_mode
+        self.subproject = subproject
         self.optional = optional
 
+class InstallDataBase:
+    def __init__(self, path: str, install_path: str, install_mode: 'FileMode', subproject: str):
+        self.path = path
+        self.install_path = install_path
+        self.install_mode = install_mode
+        self.subproject = subproject
+
+class SubdirInstallData(InstallDataBase):
+    def __init__(self, path: str, install_path: str, install_mode: 'FileMode', exclude, subproject: str):
+        super().__init__(path, install_path, install_mode, subproject)
+        self.exclude = exclude
+
 class ExecutableSerialisation:
     def __init__(self, cmd_args, env: T.Optional[build.EnvironmentVariables] = None, exe_wrapper=None,
                  workdir=None, extra_paths=None, capture=None) -> None:
         self.cmd_args = cmd_args
         self.env = env
         if exe_wrapper is not None:
-            assert(isinstance(exe_wrapper, dependencies.ExternalProgram))
+            assert(isinstance(exe_wrapper, programs.ExternalProgram))
         self.exe_runner = exe_wrapper
         self.workdir = workdir
         self.extra_paths = extra_paths
@@ -137,10 +156,11 @@ class ExecutableSerialisation:
         self.pickled = False
         self.skip_if_destdir = False
         self.verbose = False
+        self.subproject = ''
 
 class TestSerialisation:
     def __init__(self, name: str, project: str, suite: str, fname: T.List[str],
-                 is_cross_built: bool, exe_wrapper: T.Optional[dependencies.ExternalProgram],
+                 is_cross_built: bool, exe_wrapper: T.Optional[programs.ExternalProgram],
                  needs_exe_wrapper: bool, is_parallel: bool, cmd_args: T.List[str],
                  env: build.EnvironmentVariables, should_fail: bool,
                  timeout: T.Optional[int], workdir: T.Optional[str],
@@ -152,7 +172,7 @@ class TestSerialisation:
         self.fname = fname
         self.is_cross_built = is_cross_built
         if exe_wrapper is not None:
-            assert(isinstance(exe_wrapper, dependencies.ExternalProgram))
+            assert(isinstance(exe_wrapper, programs.ExternalProgram))
         self.exe_runner = exe_wrapper
         self.is_parallel = is_parallel
         self.cmd_args = cmd_args
@@ -211,6 +231,8 @@ class Backend:
         self.source_dir = self.environment.get_source_dir()
         self.build_to_src = mesonlib.relpath(self.environment.get_source_dir(),
                                              self.environment.get_build_dir())
+        self.src_to_build = mesonlib.relpath(self.environment.get_build_dir(),
+                                             self.environment.get_source_dir())
 
     def generate(self) -> None:
         raise RuntimeError('generate is not implemented in {}'.format(type(self).__name__))
@@ -247,15 +269,22 @@ class Backend:
             return self.environment.coredata.validate_option_value(option_name, override)
         return self.environment.coredata.get_option(option_name.evolve(subproject=target.subproject))
 
-    def get_source_dir_include_args(self, target, compiler):
+    def get_source_dir_include_args(self, target, compiler, *, absolute_path=False):
         curdir = target.get_subdir()
-        tmppath = os.path.normpath(os.path.join(self.build_to_src, curdir))
+        if absolute_path:
+            lead = self.source_dir
+        else:
+            lead = self.build_to_src
+        tmppath = os.path.normpath(os.path.join(lead, curdir))
         return compiler.get_include_args(tmppath, False)
 
-    def get_build_dir_include_args(self, target, compiler):
-        curdir = target.get_subdir()
-        if curdir == '':
-            curdir = '.'
+    def get_build_dir_include_args(self, target, compiler, *, absolute_path=False):
+        if absolute_path:
+            curdir = os.path.join(self.build_dir, target.get_subdir())
+        else:
+            curdir = target.get_subdir()
+            if curdir == '':
+                curdir = '.'
         return compiler.get_include_args(curdir, False)
 
     def get_target_filename_for_linking(self, target):
@@ -270,14 +299,14 @@ class Backend:
             return os.path.join(self.get_target_dir(target), target.get_filename())
         elif isinstance(target, (build.CustomTarget, build.CustomTargetIndex)):
             if not target.is_linkable_target():
-                raise MesonException('Tried to link against custom target "{}", which is not linkable.'.format(target.name))
+                raise MesonException(f'Tried to link against custom target "{target.name}", which is not linkable.')
             return os.path.join(self.get_target_dir(target), target.get_filename())
         elif isinstance(target, build.Executable):
             if target.import_filename:
                 return os.path.join(self.get_target_dir(target), target.get_import_filename())
             else:
                 return None
-        raise AssertionError('BUG: Tried to link to {!r} which is not linkable'.format(target))
+        raise AssertionError(f'BUG: Tried to link to {target!r} which is not linkable')
 
     @lru_cache(maxsize=None)
     def get_target_dir(self, target):
@@ -323,7 +352,7 @@ class Backend:
     def get_unity_source_file(self, target, suffix, number):
         # There is a potential conflict here, but it is unlikely that
         # anyone both enables unity builds and has a file called foo-unity.cpp.
-        osrc = '{}-unity{}.{}'.format(target.name, number, suffix)
+        osrc = f'{target.name}-unity{number}.{suffix}'
         return mesonlib.File.from_built_file(self.get_target_private_dir(target), osrc)
 
     def generate_unity_files(self, target, unity_src):
@@ -356,7 +385,7 @@ class Backend:
                     ofile = init_language_file(comp.get_default_suffix(), unity_file_number)
                     unity_file_number += 1
                     files_in_current = 0
-                ofile.write('#include<{}>\n'.format(src))
+                ofile.write(f'#include<{src}>\n')
                 files_in_current += 1
             if ofile:
                 ofile.close()
@@ -389,12 +418,18 @@ class Backend:
                 raise MesonException('Unknown data type in object list.')
         return obj_list
 
+    def is_swift_target(self, target):
+        for s in target.sources:
+            if s.endswith('swift'):
+                return True
+        return False
+
     def get_executable_serialisation(self, cmd, workdir=None,
                                      extra_bdeps=None, capture=None,
                                      env: T.Optional[build.EnvironmentVariables] = None):
         exe = cmd[0]
         cmd_args = cmd[1:]
-        if isinstance(exe, dependencies.ExternalProgram):
+        if isinstance(exe, programs.ExternalProgram):
             exe_cmd = exe.get_command()
             exe_for_machine = exe.for_machine
         elif isinstance(exe, build.BuildTarget):
@@ -431,7 +466,7 @@ class Backend:
         else:
             if exe_cmd[0].endswith('.jar'):
                 exe_cmd = ['java', '-jar'] + exe_cmd
-            elif exe_cmd[0].endswith('.exe') and not (mesonlib.is_windows() or mesonlib.is_cygwin()):
+            elif exe_cmd[0].endswith('.exe') and not (mesonlib.is_windows() or mesonlib.is_cygwin() or mesonlib.is_wsl()):
                 exe_cmd = ['mono'] + exe_cmd
             exe_wrapper = None
 
@@ -478,7 +513,7 @@ class Backend:
                     ['--internal', 'exe', '--capture', capture, '--'] + es.cmd_args),
                     ', '.join(reasons))
 
-        if isinstance(exe, (dependencies.ExternalProgram,
+        if isinstance(exe, (programs.ExternalProgram,
                             build.BuildTarget, build.CustomTarget)):
             basename = exe.name
         elif isinstance(exe, mesonlib.File):
@@ -493,7 +528,7 @@ class Backend:
         data = bytes(str(es.env) + str(es.cmd_args) + str(es.workdir) + str(capture),
                      encoding='utf-8')
         digest = hashlib.sha1(data).hexdigest()
-        scratch_file = 'meson_exe_{0}_{1}.dat'.format(basename, digest)
+        scratch_file = f'meson_exe_{basename}_{digest}.dat'
         exe_data = os.path.join(self.environment.get_scratch_dir(), scratch_file)
         with open(exe_data, 'wb') as f:
             pickle.dump(es, f)
@@ -563,7 +598,7 @@ class Backend:
                 for dir in symbols_match.group(1).split(':'):
                     # Prevent usage of --just-symbols to specify rpath
                     if Path(dir).is_dir():
-                        raise MesonException('Invalid arg for --just-symbols, {} is a directory.'.format(dir))
+                        raise MesonException(f'Invalid arg for --just-symbols, {dir} is a directory.')
         return dirs
 
     def rpaths_for_bundled_shared_libraries(self, target, exclude_system=True):
@@ -589,7 +624,7 @@ class Backend:
                 continue
             if libdir.startswith(self.environment.get_source_dir()):
                 rel_to_src = libdir[len(self.environment.get_source_dir()) + 1:]
-                assert not os.path.isabs(rel_to_src), 'rel_to_src: {} is absolute'.format(rel_to_src)
+                assert not os.path.isabs(rel_to_src), f'rel_to_src: {rel_to_src} is absolute'
                 paths.append(os.path.join(self.build_to_src, rel_to_src))
             else:
                 paths.append(libdir)
@@ -705,7 +740,7 @@ class Backend:
     def create_msvc_pch_implementation(self, target, lang, pch_header):
         # We have to include the language in the file name, otherwise
         # pch.c and pch.cpp will both end up as pch.obj in VS backends.
-        impl_name = 'meson_pch-{}.{}'.format(lang, lang)
+        impl_name = f'meson_pch-{lang}.{lang}'
         pch_rel_to_build = os.path.join(self.get_target_private_dir(target), impl_name)
         # Make sure to prepend the build dir, since the working directory is
         # not defined. Otherwise, we might create the file in the wrong path.
@@ -821,7 +856,7 @@ class Backend:
         args = []
         for d in deps:
             if not (d.is_linkable_target()):
-                raise RuntimeError('Tried to link with a non-library target "{}".'.format(d.get_basename()))
+                raise RuntimeError(f'Tried to link with a non-library target "{d.get_basename()}".')
             arg = self.get_target_filename_for_linking(d)
             if not arg:
                 continue
@@ -885,11 +920,11 @@ class Backend:
         arr = []
         for t in sorted(tests, key=lambda tst: -1 * tst.priority):
             exe = t.get_exe()
-            if isinstance(exe, dependencies.ExternalProgram):
+            if isinstance(exe, programs.ExternalProgram):
                 cmd = exe.get_command()
             else:
                 cmd = [os.path.join(self.environment.get_build_dir(), self.get_target_filename(t.get_exe()))]
-            if isinstance(exe, (build.BuildTarget, dependencies.ExternalProgram)):
+            if isinstance(exe, (build.BuildTarget, programs.ExternalProgram)):
                 test_for_machine = exe.for_machine
             else:
                 # E.g. an external verifier or simulator program run on a generated executable.
@@ -971,7 +1006,7 @@ class Backend:
         with open(ifilename, 'w') as f:
             f.write(json.dumps(mfobj))
         # Copy file from, to, and with mode unchanged
-        d.data.append((ifilename, ofilename, None))
+        d.data.append(InstallDataBase(ifilename, ofilename, None, ''))
 
     def get_regen_filelist(self):
         '''List of all files whose alteration means that the build
@@ -985,6 +1020,16 @@ class Backend:
         self.check_clock_skew(deps)
         return deps
 
+    def generate_regen_info(self):
+        deps = self.get_regen_filelist()
+        regeninfo = RegenInfo(self.environment.get_source_dir(),
+                              self.environment.get_build_dir(),
+                              deps)
+        filename = os.path.join(self.environment.get_scratch_dir(),
+                                'regeninfo.dump')
+        with open(filename, 'wb') as f:
+            pickle.dump(regeninfo, f)
+
     def check_clock_skew(self, file_list):
         # If a file that leads to reconfiguration has a time
         # stamp in the future, it will trigger an eternal reconfigure
@@ -999,7 +1044,7 @@ class Backend:
             # to the future by a minuscule amount, less than
             # 0.001 seconds. I don't know why.
             if delta > 0.001:
-                raise MesonException('Clock skew detected. File {} has a time stamp {:.4f}s in the future.'.format(absf, delta))
+                raise MesonException(f'Clock skew detected. File {absf} has a time stamp {delta:.4f}s in the future.')
 
     def build_target_to_cmd_array(self, bt):
         if isinstance(bt, build.BuildTarget):
@@ -1024,7 +1069,7 @@ class Backend:
             m = regex.search(arg)
             while m is not None:
                 index = int(m.group(1))
-                src = '@OUTPUT{}@'.format(index)
+                src = f'@OUTPUT{index}@'
                 arg = arg.replace(src, os.path.join(private_dir, output_list[index]))
                 m = regex.search(arg)
             newargs.append(arg)
@@ -1089,7 +1134,7 @@ class Backend:
             elif isinstance(i, build.BuildTarget):
                 fname = [self.get_target_filename(i)]
             elif isinstance(i, (build.CustomTarget, build.CustomTargetIndex)):
-                fname = [os.path.join(self.get_target_dir(i), p) for p in i.get_outputs()]
+                fname = [os.path.join(self.get_custom_target_output_dir(i), p) for p in i.get_outputs()]
             elif isinstance(i, build.GeneratedList):
                 fname = [os.path.join(self.get_target_private_dir(target), p) for p in i.get_outputs()]
             elif isinstance(i, build.ExtractedObjects):
@@ -1117,12 +1162,50 @@ class Backend:
                     deps.append(os.path.join(self.build_to_src, target.subdir, i))
         return deps
 
+    def get_custom_target_output_dir(self, target):
+        # The XCode backend is special. A target foo/bar does
+        # not go to ${BUILDDIR}/foo/bar but instead to
+        # ${BUILDDIR}/${BUILDTYPE}/foo/bar.
+        # Currently we set the include dir to be the former,
+        # and not the latter. Thus we need this extra customisation
+        # point. If in the future we make include dirs et al match
+        # ${BUILDDIR}/${BUILDTYPE} instead, this becomes unnecessary.
+        return self.get_target_dir(target)
+
+    @lru_cache(maxsize=None)
+    def get_normpath_target(self, source) -> str:
+        return os.path.normpath(source)
+
+    def get_custom_target_dirs(self, target, compiler, *, absolute_path=False):
+        custom_target_include_dirs = []
+        for i in target.get_generated_sources():
+            # Generator output goes into the target private dir which is
+            # already in the include paths list. Only custom targets have their
+            # own target build dir.
+            if not isinstance(i, (build.CustomTarget, build.CustomTargetIndex)):
+                continue
+            idir = self.get_normpath_target(self.get_custom_target_output_dir(i))
+            if not idir:
+                idir = '.'
+            if absolute_path:
+                idir = os.path.join(self.environment.get_build_dir(), idir)
+            if idir not in custom_target_include_dirs:
+                custom_target_include_dirs.append(idir)
+        return custom_target_include_dirs
+
+    def get_custom_target_dir_include_args(self, target, compiler, *, absolute_path=False):
+        incs = []
+        for i in self.get_custom_target_dirs(target, compiler, absolute_path=absolute_path):
+            incs += compiler.get_include_args(i, False)
+        return incs
+
+
     def eval_custom_target_command(self, target, absolute_outputs=False):
         # We want the outputs to be absolute only when using the VS backend
         # XXX: Maybe allow the vs backend to use relative paths too?
         source_root = self.build_to_src
         build_root = '.'
-        outdir = self.get_target_dir(target)
+        outdir = self.get_custom_target_output_dir(target)
         if absolute_outputs:
             source_root = self.environment.get_source_dir()
             build_root = self.environment.get_build_dir()
@@ -1141,10 +1224,10 @@ class Backend:
                 # GIR scanner will attempt to execute this binary but
                 # it assumes that it is in path, so always give it a full path.
                 tmp = i.get_outputs()[0]
-                i = os.path.join(self.get_target_dir(i), tmp)
+                i = os.path.join(self.get_custom_target_output_dir(i), tmp)
             elif isinstance(i, mesonlib.File):
                 i = i.rel_to_builddir(self.build_to_src)
-                if target.absolute_paths:
+                if target.absolute_paths or absolute_outputs:
                     i = os.path.join(self.environment.get_build_dir(), i)
             # FIXME: str types are blindly added ignoring 'target.absolute_paths'
             # because we can't know if they refer to a file or just a string
@@ -1153,6 +1236,8 @@ class Backend:
                     i = i.replace('@SOURCE_ROOT@', source_root)
                 if '@BUILD_ROOT@' in i:
                     i = i.replace('@BUILD_ROOT@', build_root)
+                if '@CURRENT_SOURCE_DIR@' in i:
+                    i = i.replace('@CURRENT_SOURCE_DIR@', os.path.join(source_root, target.subdir))
                 if '@DEPFILE@' in i:
                     if target.depfile is None:
                         msg = 'Custom target {!r} has @DEPFILE@ but no depfile ' \
@@ -1166,18 +1251,6 @@ class Backend:
                     else:
                         pdir = self.get_target_private_dir(target)
                     i = i.replace('@PRIVATE_DIR@', pdir)
-                if '@PRIVATE_OUTDIR_' in i:
-                    match = re.search(r'@PRIVATE_OUTDIR_(ABS_)?([^/\s*]*)@', i)
-                    if not match:
-                        msg = 'Custom target {!r} has an invalid argument {!r}' \
-                              ''.format(target.name, i)
-                        raise MesonException(msg)
-                    source = match.group(0)
-                    if match.group(1) is None and not target.absolute_paths:
-                        lead_dir = ''
-                    else:
-                        lead_dir = self.environment.get_build_dir()
-                    i = i.replace(source, os.path.join(lead_dir, outdir))
             else:
                 err_msg = 'Argument {0} is of unknown type {1}'
                 raise RuntimeError(err_msg.format(str(i), str(type(i))))
@@ -1209,10 +1282,10 @@ class Backend:
     def get_run_target_env(self, target: build.RunTarget) -> build.EnvironmentVariables:
         env = target.env if target.env else build.EnvironmentVariables()
         introspect_cmd = join_args(self.environment.get_build_command() + ['introspect'])
-        env.add_var(env.set, 'MESON_SOURCE_ROOT', [self.environment.get_source_dir()], {})
-        env.add_var(env.set, 'MESON_BUILD_ROOT', [self.environment.get_build_dir()], {})
-        env.add_var(env.set, 'MESON_SUBDIR', [target.subdir], {})
-        env.add_var(env.set, 'MESONINTROSPECT', [introspect_cmd], {})
+        env.set('MESON_SOURCE_ROOT', [self.environment.get_source_dir()])
+        env.set('MESON_BUILD_ROOT', [self.environment.get_build_dir()])
+        env.set('MESON_SUBDIR', [target.subdir])
+        env.set('MESONINTROSPECT', [introspect_cmd])
         return env
 
     def run_postconf_scripts(self) -> None:
@@ -1225,7 +1298,7 @@ class Backend:
 
         for s in self.build.postconf_scripts:
             name = ' '.join(s.cmd_args)
-            mlog.log('Running postconf script {!r}'.format(name))
+            mlog.log(f'Running postconf script {name!r}')
             run_exe(s, env)
 
     def create_install_data(self) -> InstallData:
@@ -1294,7 +1367,7 @@ class Backend:
                     i = TargetInstallData(self.get_target_filename(t), outdirs[0],
                                           t.get_aliases(), should_strip, mappings,
                                           t.rpath_dirs_to_remove,
-                                          t.install_rpath, install_mode)
+                                          t.install_rpath, install_mode, t.subproject)
                     d.targets.append(i)
 
                     if isinstance(t, (build.SharedLibrary, build.SharedModule, build.Executable)):
@@ -1312,14 +1385,15 @@ class Backend:
                             # Install the import library; may not exist for shared modules
                             i = TargetInstallData(self.get_target_filename_for_linking(t),
                                                   implib_install_dir, {}, False, {}, set(), '', install_mode,
-                                                  optional=isinstance(t, build.SharedModule))
+                                                  t.subproject, optional=isinstance(t, build.SharedModule))
                             d.targets.append(i)
 
                         if not should_strip and t.get_debug_filename():
                             debug_file = os.path.join(self.get_target_dir(t), t.get_debug_filename())
                             i = TargetInstallData(debug_file, outdirs[0],
                                                   {}, False, {}, set(), '',
-                                                  install_mode, optional=True)
+                                                  install_mode, t.subproject,
+                                                  optional=True)
                             d.targets.append(i)
                 # Install secondary outputs. Only used for Vala right now.
                 if num_outdirs > 1:
@@ -1328,7 +1402,8 @@ class Backend:
                         if outdir is False:
                             continue
                         f = os.path.join(self.get_target_dir(t), output)
-                        i = TargetInstallData(f, outdir, {}, False, {}, set(), None, install_mode)
+                        i = TargetInstallData(f, outdir, {}, False, {}, set(), None,
+                                              install_mode, t.subproject)
                         d.targets.append(i)
             elif isinstance(t, build.CustomTarget):
                 # If only one install_dir is specified, assume that all
@@ -1342,7 +1417,7 @@ class Backend:
                     for output in t.get_outputs():
                         f = os.path.join(self.get_target_dir(t), output)
                         i = TargetInstallData(f, outdirs[0], {}, False, {}, set(), None, install_mode,
-                                              optional=not t.build_by_default)
+                                              t.subproject, optional=not t.build_by_default)
                         d.targets.append(i)
                 else:
                     for output, outdir in zip(t.get_outputs(), outdirs):
@@ -1351,7 +1426,7 @@ class Backend:
                             continue
                         f = os.path.join(self.get_target_dir(t), output)
                         i = TargetInstallData(f, outdir, {}, False, {}, set(), None, install_mode,
-                                              optional=not t.build_by_default)
+                                              t.subproject, optional=not t.build_by_default)
                         d.targets.append(i)
 
     def generate_custom_install_script(self, d: InstallData) -> None:
@@ -1384,7 +1459,7 @@ class Backend:
                     msg = 'Invalid header type {!r} can\'t be installed'
                     raise MesonException(msg.format(f))
                 abspath = f.absolute_path(srcdir, builddir)
-                i = (abspath, outdir, h.get_custom_install_mode())
+                i = InstallDataBase(abspath, outdir, h.get_custom_install_mode(), h.subproject)
                 d.headers.append(i)
 
     def generate_man_install(self, d: InstallData) -> None:
@@ -1395,10 +1470,16 @@ class Backend:
                 num = f.split('.')[-1]
                 subdir = m.get_custom_install_dir()
                 if subdir is None:
-                    subdir = os.path.join(manroot, 'man' + num)
+                    if m.locale:
+                        subdir = os.path.join(manroot, m.locale, 'man' + num)
+                    else:
+                        subdir = os.path.join(manroot, 'man' + num)
+                fname = f.fname
+                if m.locale: # strip locale from file name
+                    fname = fname.replace(f'.{m.locale}', '')
                 srcabs = f.absolute_path(self.environment.get_source_dir(), self.environment.get_build_dir())
-                dstabs = os.path.join(subdir, os.path.basename(f.fname))
-                i = (srcabs, dstabs, m.get_custom_install_mode())
+                dstabs = os.path.join(subdir, os.path.basename(fname))
+                i = InstallDataBase(srcabs, dstabs, m.get_custom_install_mode(), m.subproject)
                 d.man.append(i)
 
     def generate_data_install(self, d: InstallData):
@@ -1413,7 +1494,7 @@ class Backend:
             for src_file, dst_name in zip(de.sources, de.rename):
                 assert(isinstance(src_file, mesonlib.File))
                 dst_abs = os.path.join(subdir, dst_name)
-                i = (src_file.absolute_path(srcdir, builddir), dst_abs, de.install_mode)
+                i = InstallDataBase(src_file.absolute_path(srcdir, builddir), dst_abs, de.install_mode, de.subproject)
                 d.data.append(i)
 
     def generate_subdir_install(self, d: InstallData) -> None:
@@ -1429,8 +1510,8 @@ class Backend:
                                    sd.install_dir)
             if not sd.strip_directory:
                 dst_dir = os.path.join(dst_dir, os.path.basename(src_dir))
-            d.install_subdirs.append(
-                (src_dir, dst_dir, sd.install_mode, sd.exclude))
+            i = SubdirInstallData(src_dir, dst_dir, sd.install_mode, sd.exclude, sd.subproject)
+            d.install_subdirs.append(i)
 
     def get_introspection_data(self, target_id: str, target: build.Target) -> T.List[T.Dict[str, T.Union[bool, str, T.List[T.Union[str, T.Dict[str, T.Union[str, T.List[str], bool]]]]]]]:
         '''
@@ -1455,6 +1536,8 @@ class Backend:
                     source_list += [j.absolute_path(self.source_dir, self.build_dir)]
                 elif isinstance(j, str):
                     source_list += [os.path.join(self.source_dir, j)]
+                elif isinstance(j, (build.CustomTarget, build.BuildTarget)):
+                    source_list += [os.path.join(self.build_dir, j.get_subdir(), o) for o in j.get_outputs()]
             source_list = list(map(lambda x: os.path.normpath(x), source_list))
 
             compiler = []
@@ -1481,3 +1564,37 @@ class Backend:
             }]
 
         return []
+
+    def get_devenv(self) -> build.EnvironmentVariables:
+        env = build.EnvironmentVariables()
+        extra_paths = set()
+        library_paths = set()
+        for t in self.build.get_targets().values():
+            cross_built = not self.environment.machines.matches_build_machine(t.for_machine)
+            can_run = not cross_built or not self.environment.need_exe_wrapper()
+            in_default_dir = t.should_install() and not t.get_install_dir(self.environment)[1]
+            if not can_run or not in_default_dir:
+                continue
+            tdir = os.path.join(self.environment.get_build_dir(), self.get_target_dir(t))
+            if isinstance(t, build.Executable):
+                # Add binaries that are going to be installed in bindir into PATH
+                # so they get used by default instead of searching on system when
+                # in developer environment.
+                extra_paths.add(tdir)
+                if mesonlib.is_windows() or mesonlib.is_cygwin():
+                    # On windows we cannot rely on rpath to run executables from build
+                    # directory. We have to add in PATH the location of every DLL needed.
+                    extra_paths.update(self.determine_windows_extra_paths(t, []))
+            elif isinstance(t, build.SharedLibrary):
+                # Add libraries that are going to be installed in libdir into
+                # LD_LIBRARY_PATH. This allows running system applications using
+                # that library.
+                library_paths.add(tdir)
+        if mesonlib.is_windows() or mesonlib.is_cygwin():
+            extra_paths.update(library_paths)
+        elif mesonlib.is_osx():
+            env.prepend('DYLD_LIBRARY_PATH', list(library_paths))
+        else:
+            env.prepend('LD_LIBRARY_PATH', list(library_paths))
+        env.prepend('PATH', list(extra_paths))
+        return env
