@@ -23,9 +23,10 @@ from mesonbuild.coredata import MesonException
 from . import ModuleReturnValue
 from . import ExtensionModule
 from . import get_include_args
-from ..dependencies import Dependency, InternalDependency, ExternalProgram
+from ..dependencies import Dependency, InternalDependency
 from ..interpreterbase import FeatureNew, InvalidArguments, noPosargs, noKwargs
 from ..interpreter import CustomTargetHolder
+from ..programs import ExternalProgram
 
 
 def ensure_list(value):
@@ -76,16 +77,33 @@ class HotdocTargetBuilder:
             return
 
         if isinstance(value, bool):
-            self.cmd.append(option)
+            if value:
+                self.cmd.append(option)
         elif isinstance(value, list):
             # Do not do anything on empty lists
             if value:
+                # https://bugs.python.org/issue9334 (from 2010 :( )
+                # The syntax with nargs=+ is inherently ambiguous
+                # A workaround for this case is to simply prefix with a space
+                # every value starting with a dash
+                escaped_value = []
+                for e in value:
+                    if isinstance(e, str) and e.startswith('-'):
+                        escaped_value += [' %s' % e]
+                    else:
+                        escaped_value += [e]
                 if option:
-                    self.cmd.extend([option] + value)
+                    self.cmd.extend([option] + escaped_value)
                 else:
-                    self.cmd.extend(value)
+                    self.cmd.extend(escaped_value)
         else:
-            self.cmd.extend([option, value])
+            # argparse gets confused if value(s) start with a dash.
+            # When an option expects a single value, the unambiguous way
+            # to specify it is with =
+            if isinstance(value, str):
+                self.cmd.extend([f'{option}={value}'])
+            else:
+                self.cmd.extend([option, value])
 
     def check_extra_arg_type(self, arg, value):
         value = getattr(value, 'held_object', value)
@@ -96,7 +114,7 @@ class HotdocTargetBuilder:
 
         valid_types = (str, bool, mesonlib.File, build.IncludeDirs, build.CustomTarget, build.BuildTarget)
         if not isinstance(value, valid_types):
-            raise InvalidArguments('Argument "%s=%s" should be of type: %s.' % (
+            raise InvalidArguments('Argument "{}={}" should be of type: {}.'.format(
                 arg, value, [t.__name__ for t in valid_types]))
 
     def process_extra_args(self):
@@ -264,7 +282,7 @@ class HotdocTargetBuilder:
 
         return os.path.relpath(_dir, os.path.join(self.builddir, self.subdir))
 
-    def check_forbiden_args(self):
+    def check_forbidden_args(self):
         for arg in ['conf_file']:
             if arg in self.kwargs:
                 raise InvalidArguments('Argument "%s" is forbidden.' % arg)
@@ -273,7 +291,7 @@ class HotdocTargetBuilder:
         self.include_paths[path] = path
 
     def make_targets(self):
-        self.check_forbiden_args()
+        self.check_forbidden_args()
         file_types = (str, mesonlib.File)
         self.process_known_arg("--index", file_types, mandatory=True, value_processor=self.ensure_file)
         self.process_known_arg("--project-version", str, mandatory=True)
@@ -308,6 +326,9 @@ class HotdocTargetBuilder:
 
         for path in self.include_paths.keys():
             self.cmd.extend(['--include-path', path])
+
+        if self.state.environment.coredata.get_option(mesonlib.OptionKey('werror', subproject=self.state.subproject)):
+            self.cmd.append('--fatal-warning')
         self.generate_hotdoc_config()
 
         target_cmd = self.build_command + ["--internal", "hotdoc"] + \
@@ -330,7 +351,7 @@ class HotdocTargetBuilder:
 
         install_script = None
         if install is True:
-            install_script = HotdocRunScript(self.build_command, [
+            install_script = self.state.backend.get_executable_serialisation(self.build_command + [
                 "--internal", "hotdoc",
                 "--install", os.path.join(fullname, 'html'),
                 '--name', self.name,
@@ -371,11 +392,6 @@ class HotdocTarget(build.CustomTarget):
         return res
 
 
-class HotdocRunScript(build.RunScript):
-    def __init__(self, script, args):
-        super().__init__(script, args)
-
-
 class HotDocModule(ExtensionModule):
     @FeatureNew('Hotdoc Module', '0.48.0')
     def __init__(self, interpreter):
@@ -388,7 +404,7 @@ class HotDocModule(ExtensionModule):
             from hotdoc.run_hotdoc import run  # noqa: F401
             self.hotdoc.run_hotdoc = run
         except Exception as e:
-            raise MesonException('hotdoc %s required but not found. (%s)' % (
+            raise MesonException('hotdoc {} required but not found. ({})'.format(
                 MIN_HOTDOC_VERSION, e))
 
     @noKwargs
